/* 
 * AIPlayer.java 
 * 
 * Gary Givental 
 * John Gotts 
 * Saul Rodriguez 
 * 
 * EECS 481 
 * 
 * Revision History: 
 * 
 * 10/19/97 - original implementation. 
 * 
 */ 
  
import java.util.*; 
 
/** 
 * Implements a AI player.
 * 
 * @version     1.0 
 * @author      Gary Givental 
 * @author      John Gotts 
 * @author      Saul Rodriguez 
 */ 
 
public class AIPlayer extends Player { 
 
/** 
 * Constructs an AI player.
 *
 * @param euchre	the card game.
 * @param observer	the game play observer.
 */ 
public AIPlayer (Euchre euchre, Observer observer, int position) { 
 
 	super (euchre, observer, position);
} 

/**
 * Performs a discard and a pickup in one operation.
 */
public void pickUp () {

	Card card;

	// Remove the trump card from the center panel and add it to our hand.
	pickUpTrump();
	card = strategy.findWorst(hand.getCards(), false);
	if (card == null) {
		System.out.println("AIPlayer.java: Strategy.findWorst() gave me a null card.");
		System.exit(0);
	}
	discardCard(card);
}

/**
 * Returns true if I set trump, false otherwise.
 *
 * @return	whether I set trump.
 */
public boolean callItUp () { 
	 
	Card card; 
	int i;
	int suit;
 	
 	if (dealer) {
		// I'm forced to pick up the trump card (screw the dealer).
 		if (euchre.getTable().isPassed()) {
 			suit = strategy.forceCall(hand.getCards());
			callSuit(suit);
			return true;
 		} else {
 			card = strategy.pickItUp();
			// Picks it up.
 			if (card != null) {
				call();
 				return true;
			// Turns it down.
 			} else {
				pass();
 				return false;
 			}
 		}
	// I'm not the dealer.
 	} else {
 		if (euchre.getTable().isPassed()) {
			suit = strategy.callTrump(hand.getCards());
			if (suit == Card.INVALID) {
				pass();
				return false;
			} else {
				callSuit(suit);
				return true;
			}
		} else {
			// Requests dealer to pick it up.
			if (strategy.orderUp()) {
				call();
				return true;
			// Passes.
			} else {
				pass();
				return false;
			}
		}
	}
} 

/** 
 * Plays the optimal card.
 */ 
public void playCard (int position) {

	Card card;

	card = strategy.findBest(getCards());
	if (card == null) {
		System.out.println("AIPlayer.java: Strategy.findBest() gave me a null card.");
		System.exit(0);
	}
	card.setFaceUp(true);
	super.playCard(card);
} 

}
