/*
 * Card.java
 *
 * Gary Givental
 * John Gotts
 * Saul Rodriguez
 *
 * EECS 481
 *
 * Revision History:
 *
 * 09/09/97 - original implementation.
 * 10/02/97 - numerous improvements.
 * 10/03/97 - small geometry fixes.
 * 10/19/97 - trump card, clickable.
 *
 */

import java.awt.*;

/**
 * Implements a playing card.
 *
 * @version	1.0
 * @author	Gary Givental
 * @author	John Gotts
 * @author	Saul Rodriguez
 */
public class Card extends Canvas {

public static final int INVALID = -1;
public static final int NONE = 0;
public static final int CLUB = 1;
public static final int DIAMOND = 2;
public static final int HEART = 3;
public static final int SPADE = 4;
protected static final int RADIUS = 5;
protected static final int HEIGHT = 80;
protected static final int WIDTH = 60;
protected static final int SMALLER_RADIUS = 4;
protected static final int SMALLER_HEIGHT = 64;
protected static final int SMALLER_WIDTH = 48;
protected static final String[] values = {"2", "3", "4", "5", "6", "7", "8", "9", "10", "Jack", "Queen", "King", "Ace", "Joker"};
protected String value;
protected int suit;
protected int i, j;
protected boolean faceUp = false;
protected boolean valid = false;
protected boolean smaller = false;
// Rank versus all other cards.
protected int rank;
// Rank versus cards in same suit.
protected int subRank;

/**
 * Constructs a new card of the specified suit, value, and sub-rank.  Jokers
 * are of suit NONE.
 *
 * @param suit		the suit of the card.
 * @param value		the value of the card.
 */
public Card(int suit, String value, int subRank) {

	if (suit == CLUB || suit == DIAMOND || suit == HEART || suit == SPADE)
		// Search for all but the final card (a joker).
		for (i = 0; i < values.length - 1; i++)
			if (value.compareTo(values[i]) == 0)
				valid = true;
	// Check for a joker.
	if (suit == NONE && value.compareTo(values[values.length - 1]) == 0)
		valid = true;
	if (valid) {
		this.suit = suit;
		this.value = value;
		this.subRank = subRank;
		setSize(WIDTH, HEIGHT);
	} else {
		this.suit = INVALID;
		this.value = null;
	}
}

/**
 * Constructs a new card of the given suit and value.  Jokers are of suit NONE.
 *
 * @param suit		the suit of the card.
 * @param value		the value of the card.
 */
public Card(int suit, String value) {

	this(suit, value, 0);
}

/**
 * Returns whether the card is valid.
 *
 * @return	whether the card is valid.
 */
public boolean isValid() {

	return valid;
}

/**
 * Returns the card's suit.
 *
 * @return	the card's suit.
 */
public int getSuit() {

	return suit;
}

/**
 * Returns the card's value.
 *
 * @return	the card's value.
 */
public String getValue() {

	return value;
}

/**
 * Returns whether the card is face up.
 *
 * @return	whether the card is face up.
 */
public boolean isFaceUp() {

	return faceUp;
}

/**
 * Sets whether the card is face up.
 *
 * @param faceUp	whether the card is face up.
 */
public void setFaceUp(boolean faceUp) {

	this.faceUp = faceUp;
}

/**
 * Sets the card's rank.
 *
 * @param rank  the card's rank.
 */
public void setRank(int rank) {

	this.rank = rank;
}

/**
 * Returns the card's rank.
 *
 * @return	the card's rank.
 */
public int getRank() {

        return rank;
}

/**
 * Sets the card's sub-rank.
 *
 * @param subRank	the card's sub-rank.
 */
public void setSubRank(int subRank) {

	this.subRank = subRank;
}


/**
 * Returns the card's sub-rank.
 *
 * @return      the card's sub-rank.
 */
public int getSubRank() {

        return subRank;
}

/**
 * Returns whether the card is smaller.
 *
 * @return	whether the card is smaller.
 */
public boolean isSmaller() {
	return smaller;
}

/**
 * Sets whether the card is smaller.
 *
 * @param smaller	whether the card is smaller.
 */
public void setSmaller(boolean smaller) {

	if (this.smaller != smaller) {
		this.smaller = smaller;
		if (smaller)
			setSize(SMALLER_WIDTH, SMALLER_HEIGHT);
		else
			setSize(WIDTH, HEIGHT);
	}
}

/**
 * Returns a string value describing the card.
 *
 * @return	a string value describing the card.
 */
public String getDescription () {

	String suit;

	if (! valid) {
		System.out.println("Called debug method getDescription() on an invalid card.");
		return new String();
	}
	suit = Card.getSuitDescription(getSuit());
	if (suit.compareTo("Invalid") == 0)
		return new String("Invalid");
	else if (suit.length() == 0)
		return getValue();
	else
		return new String(getValue() + " of " + suit);
}

/**
 * Returns a string value describing the card's suit.
 *
 * @return	a string value describing the card's suit.
 */
public static String getSuitDescription (int suit) {

	if (suit == SPADE)
		return new String("Spades");
	else if (suit == HEART)
		return new String("Hearts");
	else if (suit == CLUB)
		return new String("Clubs");
	else if (suit == DIAMOND)
		return new String("Diamonds");
	else if (suit == NONE)
		return new String();
	else
		return new String("Invalid");
}

/**
 * Paints the card.
 *
 * @param g	the specified Graphics window.
 */
public void paint (Graphics g) {

	FontMetrics fontMetrics = g.getFontMetrics();
	Color oldColor = g.getColor();
	Color suitColor;
	String suitSymbol;
	int fontAscent = fontMetrics.getAscent();
	int fontDescent = fontMetrics.getDescent();
	int stringWidth, radius, height, width;

	if (valid) {
		if (smaller) {
			radius = SMALLER_RADIUS;
			height = SMALLER_HEIGHT;
			width = SMALLER_WIDTH;
		} else {
			radius = RADIUS;
			height = HEIGHT;
			width = WIDTH;
		}
		if (faceUp) {
			g.setColor(Color.white);
			g.fillRoundRect(0, 0, width - 1, height - 1, radius,
				radius);
			g.setColor(Color.black);
			g.drawRoundRect(0, 0, width - 1, height - 1, radius,
				radius);
			// Eventually, we will draw a symbol for the suit, but a
			// character will suffice for now.
			if (suit == SPADE) {
				suitSymbol = new String("S");
				suitColor = Color.black;
			} else if (suit == CLUB) {
				suitSymbol = new String("C");
				suitColor = Color.black;
			} else if (suit == DIAMOND) {
				suitSymbol = new String("D");
				suitColor = Color.red;
			} else if (suit == HEART) {
				suitSymbol = new String("H");
				suitColor = Color.red;
			} else { // suit == NONE
				suitSymbol = new String("X");
				suitColor = Color.black;
			}
			g.setColor(suitColor);
			stringWidth = fontMetrics.stringWidth(suitSymbol);
			g.drawString(suitSymbol, (width - stringWidth) / 2,
				(height + fontAscent - fontDescent) / 2);
			stringWidth = fontMetrics.stringWidth(value);
			// The values are spaced width / 20 and height / 20
			// pixels away from the edges.  Neglect the thickness
			// of the baseline.
			g.drawString(value, width / 20, height / 20 +
				fontAscent);
			g.drawString(value, width - stringWidth - width /
				20, height - fontDescent - height / 20);
		} else { // Face down.
			// An image might eventually go here.
			g.setColor(Color.black);
			g.fillRoundRect(0, 0, width - 1, height - 1, radius,
				radius);
			g.drawRoundRect(0, 0, width - 1, height - 1, radius,
				radius);
		}
		g.setColor(oldColor);
	}
}

}
