/*
 * Console.java
 *
 * Gary Givental
 * John Gotts
 * Saul Rodriguez
 *
 * EECS 481
 *
 * Revision History:
 *
 * 11/06/97 - original implementation.
 * 11/28/97 - reset() method.
 *            cleaned up.
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * Implements a console.  This window serves as the chat/status window.
 *
 * @version     1.0
 * @author      Gary Givental
 * @author      John Gotts
 * @author      Saul Rodriguez
 */
public class Console extends Frame implements ActionListener {

protected GUI gui;
protected Observer observer;
protected TextArea textArea;
protected TextField textField;

/**
 * Constructs a console.
 *
 * @param euchre	the card game.
 */
public Console (Euchre euchre) {

	super("Java Euchre Console");
	this.observer = euchre.getObserver();
	this.gui = euchre.getGUI();
	textArea = new TextArea(20, 80);
	textArea.setEditable(false);
	add ("North", textArea);
	textField = new TextField ("", 80);
	textField.addActionListener(this);
	add ("Center", textField);
	pack();
}

/**
 * Handles user input.
 *
 * @param event	the event.
 */
public void actionPerformed(ActionEvent event) {

	Object source = event.getSource();
	StringTokenizer stringTokenizer;
	TextField textField;
	String text;

	// Added in case any other component is added later.
	if (source instanceof TextField) {
		textField = (TextField) source;
		text = textField.getText();
		if (text.length() > 0) {
			textField.setText(new String());
			stringTokenizer = new StringTokenizer(text);
			// Just in case...
			if (stringTokenizer.hasMoreTokens()) {
				if (stringTokenizer.nextToken().compareTo("%") == 0)
					observer.debug(text);
				else
					observer.processOutgoingText("S " + text);
			}
		}
	}
}

/**
 * Adds the specified text to the text area.
 *
 * @param text	the text to add to the text area.
 */
public void addText(String text) {

	textArea.append(text + "\n");
}

/**
 * Shows or hides the console depending on the value of parameter b.  This
 * method also forces the console to appear to the right of the GUI.
 *
 * @param b	if true, it shows the console; otherwise, it hides it.
 */
public void setVisible(boolean b) {

	Rectangle rectangle;
	Rectangle rectangle2;
	Point point;

	if (b) {
		// Caveat: This sort of thing is generally frowned upon and
		// may not even work in some environments.
		rectangle = gui.getBounds();
		rectangle2 = getBounds();
		point = new Point(rectangle.x + rectangle.width, rectangle.y + (rectangle.height - rectangle2.height) / 2);
		super.setVisible(b);
		setLocation(point);
	} else
		super.setVisible(b);
}

/**
 * Resets the console.  Currently, this method need not do anything.
 */
public void reset () {

}

}
