/*
 * Deck.java
 *
 * Gary Givental
 * John Gotts
 * Saul Rodriguez
 *
 * EECS 481
 *
 * Revision History:
 *
 * 10/02/97 - original implementation.
 * 11/28/97 - overhauled.
 *
 */

import java.util.*;

/**
 * Implements a deck of cards.
 *
 * @version     1.0
 * @author      Gary Givental
 * @author      John Gotts
 * @author      Saul Rodriguez
 */
public class Deck {

public static final int INVALID = -1;
public static final int EUCHRE = 0;
protected static Random random = new Random();
protected Vector gameCards;
protected Vector cards;
protected int gameType = INVALID;
protected boolean valid = true;

/**
 * Constructs a deck of cards for the specified card game.
 *
 * @param gameType	the type of card game.
 */
public Deck (int gameType) {

	if (gameType == EUCHRE) {
		this.gameType = gameType;
		setGameCards();
		reset();
	} else {
		valid = false;
		System.out.println("Deck.java: Game type " + gameType + " not implemented.");
	}
}

/**
 * Sets the game cards based upon the type of card game.
 */
protected void setGameCards () {

	if (gameType == EUCHRE) {
		gameCards = new Vector();
		gameCards.addElement(new NameValuePair ("9", 0));
		gameCards.addElement(new NameValuePair ("10", 1));
		gameCards.addElement(new NameValuePair ("Jack", 2));
		gameCards.addElement(new NameValuePair ("Queen", 3));
		gameCards.addElement(new NameValuePair ("King", 4));
		gameCards.addElement(new NameValuePair ("Ace", 5));
	} // No other possibilities for the moment.
}

/**
 * Returns whether the deck is valid.
 *
 * @return	whether the card is valid.
 */
public boolean isValid() {

	return valid;
}

/**
 * Adds two cards for the joker and four cards (a club, a diamond, a heart,
 * and a spade) for every other card specified in the vector of name-value
 * pairs.
 *
 * @param cards	the cards to be added to the deck.
 */
protected void addCards (Vector gameCards) {

	NameValuePair pair;
	Card card;
	Enumeration e;
	String value;
	int i, subRank;

	for (e = gameCards.elements(); e.hasMoreElements();) {
		pair = (NameValuePair) e.nextElement();
		value = pair.getName();
		subRank = pair.getValue();
		// Don't check whether a card's been specified more than once
		// to allow for card games that use multiple physical decks.
		if (value.compareTo("Joker") == 0) {
			for (i = 0; i < 2; i++) {
				card = new Card(Card.NONE, value, subRank);
				if (card.isValid())
					cards.addElement(card);
				else
					valid = false;
			}
		} else {
			card = new Card(Card.CLUB, value, subRank);
			if (card.isValid())
				cards.addElement(card);
			else
				valid = false;
			card = new Card(Card.DIAMOND, value, subRank);
			if (card.isValid())
				cards.addElement(card);
			else
				valid = false;
			card = new Card(Card.HEART, value, subRank);
			if (card.isValid())
				cards.addElement(card);
			else
				valid = false;
			card = new Card(Card.SPADE, value, subRank);
			if (card.isValid())
				cards.addElement(card);
			else
				valid = false;
		}
	}
}

/**
 * Draws (deals) a card from the deck.  This (not surprisingly) removes that
 * card from the deck.
 *
 * @return	the card.
 */
public Card drawCard () {

	Card card;
	int i;

	// Generate a random number between 0 and cards.size() - 1, inclusive.
	i = (int) (random.nextDouble() * (double) cards.size());
	card = (Card) cards.elementAt(i);
	cards.removeElementAt(i);
	return card;
}

/**
 * Resets the deck.
 */
public void reset() {

	cards = new Vector();
	addCards(gameCards);
}

}
