/*
 * Euchre.java
 *
 * Gary Givental
 * John Gotts
 * Saul Rodriguez
 *
 * EECS 481
 *
 * Revision History:
 *
 * 09/09/97 - original implementation.
 * 11/28/97 - updated Deck constructor.
 *            players, network moved to the observer class.
 *
 * The euchre class instantiates objects that persist throughout the entire
 * game.  For maximum simplicity, all of the objects that the euchre class
 * creates communicate with each other through it.  These objects all
 * implement a reset() method to facilitate object reuse.
 *
 */

import java.awt.*;
import java.util.*;

/**
 * Implements the Euchre card game.
 *
 * @version     1.0
 * @author      Gary Givental
 * @author      John Gotts
 * @author      Saul Rodriguez
 */
public class Euchre {

protected Observer observer;
protected Table table;
protected GUI gui;
protected Console console;
protected Deck deck;

/**
 * Constructs a Euchre card game.
 */
public Euchre () {

	observer = new Observer(this);
	table = new Table(this);
	gui = new GUI(this);
	console = new Console(this);
	gui.setVisible(true);
	console.setVisible(true);
	deck = new Deck(Deck.EUCHRE);
}

/**
 * Returns the observer.
 *
 * @return	the observer.
 */
public Observer getObserver () {

	return observer;
}

/**
 * Returns the table.
 *
 * @return	the table.
 */
public Table getTable() {

	return table;
}

/**
 * Returns the GUI.
 *
 * @return	the GUI.
 */
public GUI getGUI() {

    return gui;
}

/**
 * Returns the console.
 *
 * @return	the console.
 */
public Console getConsole() {

	return console;
}

/**
 * Returns the deck.
 *
 * @return	the deck.
 */
public Deck getDeck() {

	return deck;
}

/**
 * Starts the game.
 */
public static void main(String args[]) {

	Euchre euchre = new Euchre();
}

}
