/*
 * GUI.java
 *
 * Gary Givental
 * John Gotts
 * Saul Rodriguez
 *
 * EECS 481
 *
 * Revision History:
 *
 * 09/09/97 - original implementation.
 * 10/05/97 - readied the code for the presentation.
 * 10/19/97 - add ability to move cards, trump card
 * 11/29/97 - optimizations.
 * 12/06/97 - lots of new panels with new information.
 *
 */

import java.awt.*;
import java.util.*;

/**
 * Implements the euchre GUI.
 *
 * @version     1.0
 * @author      Gary Givental
 * @author      John Gotts
 * @author      Saul Rodriguez
 */
public class GUI extends Frame
{
    protected Euchre euchre;
    protected Observer observer;
    protected EuchreMenuBar menuBar;
    protected EuchrePanel centerPanel, northPanel, southPanel, eastPanel, westPanel;
    protected EuchrePanel centerCenterPanel, centerNorthPanel, centerSouthPanel, centerEastPanel, centerWestPanel;
    protected EuchrePanel northWestPanel, northCenterPanel, northEastPanel;
    protected EuchrePanel westNorthPanel, westCenterPanel, westSouthPanel;
    protected EuchrePanel eastNorthPanel, eastCenterPanel, eastSouthPanel;
    protected Card activeCard = null;
    protected String[] playerNames = new String[4];
    // True if the player is a human player.
    protected boolean[] playerTypes = new boolean[4];
    protected Label dealer, leader, trumpCaller, trump;
    protected Label ourScore, theirScore, ourTricks, theirTricks;

    /**
     * Constructs the euchre GUI.
     *
     * @param euchre	the card game.
     */
    public GUI (Euchre euchre) 
    {
        super("Java Euchre");

        this.euchre = euchre;
        observer = euchre.getObserver();

        // The panel that contains the trump card and the cards that are
	// played.
        centerPanel = new EuchrePanel (this, new Dimension (300, 300));
	centerPanel.setLayout(new BorderLayout());
        centerCenterPanel = new EuchrePanel (this, new Dimension (100, 100));
	centerPanel.add(centerCenterPanel);
        centerNorthPanel = new EuchrePanel (this, new Dimension (300, 100));
	centerPanel.add("North", centerNorthPanel);
        centerSouthPanel = new EuchrePanel (this, new Dimension (300, 100));
	centerPanel.add("South", centerSouthPanel);
        centerEastPanel = new EuchrePanel (this, new Dimension (100, 100));
	centerPanel.add("East", centerEastPanel);
        centerWestPanel = new EuchrePanel (this, new Dimension (100, 100));
	centerPanel.add("West", centerWestPanel);
        add (centerPanel);
	// The panel that contains the player's cards.
        southPanel = new EuchrePanel (this, new Dimension (600, 150));
        add ("South", southPanel);
	// The panel that contains an icon of player 2.
        eastPanel = new EuchrePanel (this, new Dimension (150, 300));
	eastPanel.setLayout(new BorderLayout());
        eastNorthPanel = new EuchrePanel (this, new Dimension (150, 100));
	eastPanel.add("North", eastNorthPanel);
        eastCenterPanel = new EuchrePanel (this, new Dimension (150, 100));
	eastPanel.add(eastCenterPanel);
        eastSouthPanel = new EuchrePanel (this, new Dimension (150, 100));
	eastPanel.add("South", eastSouthPanel);
        add ("East", eastPanel);
	// The panel that contains an icon of player 3.
        northPanel = new EuchrePanel (this, new Dimension(600, 150));
	northPanel.setLayout(new BorderLayout());
        northWestPanel = new EuchrePanel (this, new Dimension (200, 150));
	northWestPanel.setLayout(new GridLayout(4, 1));
	northPanel.add("West", northWestPanel);
        northCenterPanel = new EuchrePanel (this, new Dimension (200, 150));
	northPanel.add(northCenterPanel);
        northEastPanel = new EuchrePanel (this, new Dimension (200, 150));
	northEastPanel.setLayout(new GridLayout(4, 1));
	northPanel.add("East", northEastPanel);
        add ("North", northPanel);
	// The panel that contains an icon of player 4.
        westPanel = new EuchrePanel (this, new Dimension (150, 300));
	westPanel.setLayout(new BorderLayout());
        westNorthPanel = new EuchrePanel (this, new Dimension (150, 100));
	westPanel.add("North", westNorthPanel);
        westCenterPanel = new EuchrePanel (this, new Dimension (150, 100));
	westPanel.add(westCenterPanel);
        westSouthPanel = new EuchrePanel (this, new Dimension (150, 100));
	westPanel.add("South", westSouthPanel);
        add ("West", westPanel);

        // Add the menu bar.
        menuBar=new EuchreMenuBar(euchre, this);
        setMenuBar(menuBar);

        pack();

	playerNames[Observer.SOUTH] = new String();
    }

    /**
     * Returns the player name at the specified position.
     *
     * @param position	the position of the player.
     * @return	the player name at the specified position.
     */
    public String getPlayerName (int position) {

        return playerNames[position];
    }

    /**
     * Performs the necessary work when a user selects "Setup Game".
     */
    public void setupGame()
    {
        NameDialog nameDialog;

	playerNames[Observer.SOUTH] = null;
        while (playerNames[Observer.SOUTH] == null || playerNames[Observer.SOUTH].length() == 0)
        {
            nameDialog=new NameDialog(this);
            nameDialog.setVisible(true);
            playerNames[Observer.SOUTH]=nameDialog.getName();
        } // keep asking until get a name

        setTitle("Java Euchre - " + playerNames[Observer.SOUTH] + "'s Game");

        playerTypes[Observer.SOUTH] = true;
        observer.addHumanPlayer(Observer.SOUTH);

	// Add three AI players that can be replaced by network playes.

        modifyPlayer("Computer", Observer.WEST, false);
        observer.addAIPlayer(Observer.WEST);

        modifyPlayer("Computer", Observer.NORTH, false);
        observer.addAIPlayer(Observer.NORTH);
    
        modifyPlayer("Computer", Observer.EAST, false);
        observer.addAIPlayer(Observer.EAST);

	setupStats();

        pack();

        // Enable options menu.
        menuBar.enableOptions();
    }

    public void setupStats () {

        dealer = new Label("Dealer:");
        northWestPanel.removeAll();
        northWestPanel.add(dealer);
        leader = new Label("Leader:");
        northWestPanel.add(leader);
        trumpCaller = new Label("Trump Caller:");
        northWestPanel.add(trumpCaller);
        trump = new Label("Trump:");
        northWestPanel.add(trump);
        ourScore = new Label("Our Score: 0");
        northEastPanel.removeAll();
        northEastPanel.add(ourScore);
        theirScore = new Label("Their Score: 0");
        northEastPanel.add(theirScore);
        ourTricks = new Label("Our Tricks: 0");
        northEastPanel.add(ourTricks);
        theirTricks = new Label("Their Tricks: 0");
        northEastPanel.add(theirTricks);
    }

    /**
     * Actually starts the game.  Can be called by someone across the network
     * if the network is enabled.
     */
    public void playGame ()
    {
        observer.playGame(false);
    }

    /**
     * Modifies player icons.
     *
     * @param name		the player's name.
     * @param position		the player's position.
     * @param humanPlayer	whether the player's a human player.
     */
    public void modifyPlayer (String name, int position, boolean humanPlayer) {

        EuchrePanel euchrePanel = null;
        ImagePanel imagePanel;
        Panel panel;
	int playerNumber = 0;

        if (position == Observer.WEST) {
            playerNames[Observer.WEST] = name;
            playerNumber = 2;
            euchrePanel = westCenterPanel;
        } else if (position == Observer.NORTH) {
            playerNames[Observer.NORTH] = name;
            playerNumber = 3;
            euchrePanel = northCenterPanel;
        } else if (position == Observer.EAST) {
            playerNames[Observer.EAST] = name;
            playerNumber = 4;
            euchrePanel = eastCenterPanel;
        } else {
            System.out.println("GUI.java: Tried to modify myself or a non-existant player.");
            System.exit(0);
        }
        playerTypes[position] = humanPlayer;
        // The garbage collector should eliminate the objects we remove as
        // they will become unreferenced.
        euchrePanel.removeAll();
        if (humanPlayer)
            imagePanel = new ImagePanel("man.gif");
        else
            imagePanel = new ImagePanel("robotman.gif");
        panel = new Panel();
        panel.setLayout(new BorderLayout());
        panel.add("North", imagePanel);
        panel.add("South", new Label(new String(name + "(" + playerNumber + ")")));
        euchrePanel.add(panel);
    }

    /**
     * Updates the dealer.
     */
    public void updateDealer (int position) {

        int playerNumber = position + 1;

        this.dealer.setText(new String("Dealer: " + playerNumber));
    }

    /**
     * Removes the dealer.
     */
    public void removeDealer () {

        dealer.setText(new String("Dealer:"));
    }

    /**
     * Updates the leader.
     */
    public void updateLeader (int position) {

        int playerNumber = position + 1;

        this.leader.setText(new String("Leader: " + playerNumber));
    }

    /**
     * Removes the leader.
     */
    public void removeLeader () {

        leader.setText(new String("Leader:"));
    }

    /**
     * Updates the trump caller.
     */
    public void updateTrumpCaller (int position) {

        int playerNumber = position + 1;

        this.trumpCaller.setText(new String("Trump Caller: " + playerNumber));
    }

    /**
     * Removes the trump caller.
     */
    public void removeTrumpCaller () {

        trumpCaller.setText(new String("Trump Caller:"));
    }

    /**
     * Updates trump.
     *
     * @param trump	trump.
     */
    public void updateTrump (String trump) {

        this.trump.setText(new String("Trump: " + trump));
    }

    /**
     * Removes the trump suit display.
     */
    public void removeTrump () {

        trump.setText(new String("Trump:"));
    }

    /**
     * Updates our score.
     *
     * @param score	our score.
     */
    public void updateOurScore (int score) {

        ourScore.setText(new String("Our Score: " + score));
    }

    /**
     * Updates their score.
     *
     * @param score	their score.
     */
    public void updateTheirScore (int score) {

        theirScore.setText(new String("Their Score: " + score));
    }

    /**
     * Removes the scores.
     */
    public void removeScores () {

        ourScore.setText(new String("Our Score: 0"));
        theirScore.setText(new String("Their Score: 0"));
    }

    /**
     * Updates our tricks.
     *
     * @param score	our score.
     */
    public void updateOurTricks (int tricks) {

        ourTricks.setText(new String("Our Tricks: " + tricks));
    }

    /**
     * Updates their tricks.
     *
     * @param score	their score.
     */
    public void updateTheirTricks (int tricks) {

        theirTricks.setText(new String("Their Tricks: " + tricks));
    }

    /**
     * Removes the tricks.
     */
    public void removeTricks () {

        ourTricks.setText(new String("Our Tricks: 0"));
        theirTricks.setText(new String("Their Tricks: 0"));
    }

    /**
     * Allows the user to connect to the network.
     *
     * @return	whether the network started.
     */
    public boolean queryNetworkedGame()
    {
        NetworkDialog networkDialog;
        String portValue,serverValue;
        Network network;
	boolean status;

        networkDialog=new NetworkDialog(this);
        networkDialog.setVisible(true);

        portValue=networkDialog.getPort();
        serverValue=networkDialog.getServer();

        if (portValue != null && serverValue != null) {
            status = observer.startNetwork(serverValue, portValue);
	    return status;
	} else
	    return false;
	
    }

    public void advise (String subject, String message)
    {
//        InfoDialog infoDialog;

//	infoDialog = new InfoDialog(this, subject, message);
//	infoDialog.setVisible(true);
        euchre.getConsole().addText(message);
    }

    public void blockAdvise(String subject, String message)
    {
        InfoDialog infoDialog;

	infoDialog = new InfoDialog(this, subject, message, true);
	infoDialog.setVisible(true);
    }

    /**
     * Returns whether the user wants the dealer to pick up the trump card.
     *
     * @return whether the user wants the dealer to pick up the trump card.
     */
    public boolean queryPickUp () {

        YesNoDialog trumpDialog;

        trumpDialog = new YesNoDialog(this, "Pick up Trump","Pick it up?");
        trumpDialog.setVisible(true);
        return trumpDialog.getChoice();
    }

    /**
     * Returns the suit the user wants for trump.
     *
     * @return the suit the user wants for trump.
     */
    public int querySuit () {

        TrumpSuitDialog trumpSuitDialog;
        String suit;

        trumpSuitDialog = new TrumpSuitDialog(euchre, this);
        trumpSuitDialog.setVisible(true);
        return trumpSuitDialog.getSuit();
    }

		/**
     * select option for screw the dealer
     *
     * @select option for screw the dealer
     */
    public boolean screwDealer () {

        YesNoDialog screwDealerDialog;

        screwDealerDialog = new YesNoDialog(this, "Screw Dealer Option","Screw the Dealer?");
        screwDealerDialog.setVisible(true);
        return screwDealerDialog.getChoice();
    }

    /**
     * select option for screw the dealer
     *
     * @select option for screw the dealer
     */
    public boolean tips () {

        YesNoDialog tipsDialog;

        tipsDialog = new YesNoDialog(this, "Tips Option","Enable Tips?");
        tipsDialog.setVisible(true);
        return tipsDialog.getChoice();
    }

    /**
     * Adds a card to a panel.
     *
     * @param card		the card.
     * @param position		the card's position.
     * @param subPosition	the card's subposition.
     */
    public void addCard(Card card, int position, int subPosition)
    {
        if (position == Observer.SOUTH) {
            southPanel.add(card);
            card.addMouseListener(southPanel);
        } else if (position == Observer.CENTER) {
	    if (subPosition == Observer.SOUTH)
            	centerSouthPanel.add(card);
	    else if (subPosition == Observer.NORTH)
            	centerNorthPanel.add(card);
	    else if (subPosition == Observer.EAST)
            	centerEastPanel.add(card);
	    else if (subPosition == Observer.WEST)
            	centerWestPanel.add(card);
	    else if (subPosition == Observer.CENTER)
            	centerCenterPanel.add(card);
        } else {
            System.out.println("GUI.java: Tried to add a card to an invalid panel.");
            System.exit(0);
        }
	pack();
    }

    /**
     * Removes a card from a panel.
     *
     * @param card	the card.
     * @param position	the card's position.
     */
    public void removeCard(Card card, int position)
    {
        if (position == Observer.SOUTH) {
            card.removeMouseListener(southPanel);
            southPanel.remove(card);
        } else if (position == Observer.CENTER) {
            card.getParent().remove(card);
        } else {
            System.out.println("GUI.java: Tried to remove a card from an invalid panel.");
            System.exit(0);
        }
	pack();
    }

    public Card getActiveCard() {

	Card card;

	card = activeCard;
	activeCard = null;
        return card;
    }

    public void setActiveCard (Card activeCard) {

        this.activeCard = activeCard;
	// We got a card, now go back to the game.
	observer.playGame(true);
    }
    
    
 // Brings up the help dialog - Euchre Rules   
public void euchreRules()
    {
         RulesDialog rulesDialog;
 
       
       rulesDialog=new RulesDialog(this,euchre);
       rulesDialog.setVisible(true);
            
    }

// Displays the message about the game
public void helpAbout()
    {
         InfoDialog helpAboutDialog;
         String message;
 
       message=new String("Java Euchre v.1.0\n by Gary Givental\n John Gotts\n Saul Rodriguez\n");
       helpAboutDialog=new InfoDialog(this,"About Euchre",message);
       helpAboutDialog.setVisible(true);
            
    }

    public void enablePlayGame () {

        menuBar.enablePlayGame();
    }

    /**
     * Determines the insets of the GUI, which indicate the size of its
     * border. 
     *
     * @return	the insets of the GUI.
     */
    public Insets getInsets() 
    {
        Insets insets = super.getInsets();
    
        return new Insets(insets.top + 10, insets.left + 10, insets.bottom + 10, insets.right + 10);

    }
}
