/*
 * Hand.java
 *
 * Gary Givental
 * John Gotts
 * Saul Rodriguez
 *
 * EECS 481
 *
 * Revision History:
 *
 * 11/20/97 - original implementation.
 *
 */

import java.util.*;

/**
 * Implements a hand of cards.
 *
 * @version     1.0
 * @author      Gary Givental
 * @author      John Gotts
 * @author      Saul Rodriguez
 */
public class Hand {

protected Vector cards;
protected Vector playedCards;

/**
 * Constructs a hand of cards.
 */
public Hand () {

	cards = new Vector();
	playedCards = new Vector();
}

/**
 * Constructs a hand of cards with the specified cards.
 *
 * @param cards	the initial contents of the hand.
 */
public Hand (Vector cards) {

	this();
	this.cards = cards;
}

/**
 * Resets my hand.
 */
public void reset () {

	if (cards.size() > 0)
		cards = new Vector();
	if (playedCards.size() > 0)
		playedCards = new Vector();
}

/**
 * Returns true if the card was successfully played (moved from the cards
 * vector to the playedCards vector).
 *
 * @return	whether the card was successfully played.
 */
public boolean playCard (Card card) {

	int i;

	i = cards.indexOf(card);
	if (i == -1)
		return false;
	else {
		playedCards.addElement(cards.elementAt(i));
		cards.removeElementAt(i);
		return true;
	}
}

/**
 * Returns the cards contained in this hand.
 *
 * @return	the cards contained in this hand.
 */
public Vector getCards () {

	return cards;
}

/**
 * Returns the cards that have been played.
 *
 * @return	the cards that have been played.
 */
public Vector getPlayedCards () {

	return playedCards;
}

/**
 * Adds a card to this hand.
 *
 * @param card	the card to be added.
 */
public void addCard (Card card) {

	cards.addElement(card);
}

/**
 * Removes a card from this hand.
 *
 * @param card	the card to be removed.
 */
public void removeCard (Card card) {

	if (! cards.contains(card)) {
		System.out.println("Card not found in hand.");
		System.exit(0);
	}
	System.out.println("Removing " + card.getDescription() + " from hand.");
	cards.removeElement(card);
}

}
