/* 
 * HumanPlayer.java 
 * 
 * Gary Givental 
 * John Gotts 
 * Saul Rodriguez 
 * 
 * EECS 481 
 * 
 * Revision History: 
 * 
 * 10/19/97 - original implementation. 
 *
 */ 
  
import java.util.*; 
 
/** 
 * Implements a human player.
 * 
 * @version     1.0 
 * @author      Gary Givental 
 * @author      John Gotts 
 * @author      Saul Rodriguez 
 */ 
 
public class HumanPlayer extends Player { 
 
/** 
 * Constructs a human player.
 *
 * @param euchre	the card game.
 * @param observer	the game play observer.
 */ 
public HumanPlayer (Euchre euchre, Observer observer, int position) { 
 
 	super (euchre, observer, position);
} 

/**
 * Picks up the trump card.
 *
 * @return	the trump card.
 */
public Card pickUpTrump () {

	Card trumpCard;

	// Remove the trump card from the center panel, add it to our hand,
	trumpCard = super.pickUpTrump();
	// and add it to the bottom panel.
	euchre.getGUI().addCard(trumpCard, Observer.SOUTH, Observer.CENTER);
	return trumpCard;
}

/**
 * Discards the selected card.
 */
public void discardCard () {

	Card card;

	card = euchre.getGUI().getActiveCard();
	euchre.getGUI().removeCard(card, Observer.SOUTH);
	super.discardCard(card);
}

/**
 * Returns true if I set trump, false otherwise.
 *
 * @return      whether I set trump.
 */
public boolean callItUp () {

	boolean choice;
        int suit = Card.INVALID;

	if (dealer) {
		// Force the user to pick up the trump card (screw the dealer).
		if (euchre.getTable().isPassed()) {
                        while (suit == Card.INVALID) {
				suit = euchre.getGUI().querySuit();
				if (suit == Card.INVALID)
					euchre.getGUI().blockAdvise("You're Screwed", "Sorry, screw the dealer is in effect.");
			}
			callSuit(suit);
			return true;
		} else {
			choice = euchre.getGUI().queryPickUp();
			if (choice)
				call();
			else
				pass();
			return choice;
		}
	// I'm not the dealer.
	} else {
		if (euchre.getTable().isPassed()) {
			suit = euchre.getGUI().querySuit();
			if (suit == Card.INVALID) {
				pass();
				return false;
			} else {
				callSuit(suit);
				return true;
			}
		} else {
			choice = euchre.getGUI().queryPickUp();
			if (choice)
				call();
			else
				pass();
			return choice;
		}
	}
}

/** 
 * Plays the optimal card.
 *
 * @return	whether the card was legal.
 */
public boolean playCard (int position) { 

	Card card;

	card = euchre.getGUI().getActiveCard();
	if (! strategy.legalPlay(card)) return false;
	euchre.getGUI().removeCard(card, Observer.SOUTH);
	super.playCard(card);
	return true;
}

/**
 *
 */
public Card suggestPickItUp () {

	return strategy.pickItUp ();
}

/**
 * Returns the best discard candidate.
 *
 * @return	the best discard candidate.
 */
public Card suggestDiscard () {

	return strategy.findWorst(hand.getCards(), false);
}

/**
 * Returns the optimal card.
 *
 * @return	the optimal card.
 */
public Card suggestCard () {

	return strategy.findBest(getCards());
}
 
}
