/*
 * Network.java
 *
 * Gary Givental
 * John Gotts
 * Saul Rodriguez
 *
 * EECS 481
 *
 * Revision History:
 *
 * 11/05/97 - original implementation.
 * 11/28/97 - small hacks.
 *
 * This class is a thread because messages can come across the network at any
 * time.
 *
 */

import java.net.*;
import java.io.*;

/**
 * Implements a network helper class.
 *
 * @version     1.0
 * @author      Gary Givental
 * @author      John Gotts
 * @author      Saul Rodriguez
 */
public class Network extends Thread {

protected Observer observer;
protected InputStream inputStream;
protected OutputStream outputStream;
protected Socket socket;
protected boolean valid = true;

/**
 * Constructs a network connection with the specified server on the specified
 * port.
 *
 * @param server	the server.
 * @param port		the port.
 */
public Network (Observer observer, String server, int port) {

	this.observer = observer;
	try {
		socket = new Socket(server, port);
	} catch (UnknownHostException exception) {
		System.out.println("Network.java: Unknown host exception: " + exception + ".");
		valid = false;
	} catch (IOException exception) {
		System.out.println("Network.java: IO exception: " + exception + ".");
		valid = false;
	} catch (Exception exception) {
		System.out.println("Network.java: Unknown exception: " + exception + ".");
		valid = false;
	}
	if (valid) {
		try {
			inputStream = socket.getInputStream();
		} catch (IOException exception) {
			System.out.println("Network.java: IO exception: " + exception + ".");
			valid = false;
		} catch (Exception exception) {
			System.out.println("Network.java: Unknown exception: " + exception + ".");
			valid = false;
		}
	}
	if (valid) {
		try {
			outputStream = socket.getOutputStream();
		} catch (IOException exception) {
			System.out.println("Network.java: IO exception: " + exception + ".");
			valid = false;
		} catch (Exception exception) {
			System.out.println("Network.java: Unknown exception: " + exception + ".");
			valid = false;
		}
	}
}

/**
 * Constructs a network connection with the specified server on the default
 * port.
 *
 * @param server	the server.
 */
public Network (Observer observer, String server) {

	this(observer, server, 5000);
}

/**
 * Constructs a network connection with the default server on the specified
 * port.
 *
 * @param port		the port.
 */
public Network (Observer observer, int port) {

	this(observer, "localhost", port);
}

/**
 * Constructs a network connection with the default server on the default port.
 */
public Network (Observer observer) {
	this(observer, "localhost", 5000);
}

/**
 * Returns whether the network connection is valid.
 *
 * @return	whether the network connection is valid.
 */
public boolean isValid() {

	return valid;
}

/**
 * Closes the input and output streams.
 */
public void closeStreams() {

	// We don't act on the exceptions because this method is part of the
	// shutdown process.
	try {
		inputStream.close();
	} catch (IOException exception) {
		System.out.println("Network.java: IO exception: " + exception + ".");
	} catch (Exception exception) {
		System.out.println("Network.java: Unknown exception: " + exception + ".");
	}
	try {
		outputStream.close();
	} catch (IOException exception) {
		System.out.println("Network.java: IO exception: " + exception + ".");
	} catch (Exception exception) {
		System.out.println("Network.java: Unknown exception: " + exception + ".");
	}
}

/**
 * Sends a message to the server.  This method appends a newline character to
 * the message.
 *
 * @param message	the message to send.
 */
public void sendMessage (String message) {

	String tempMessage;

	tempMessage = new String (message + "\n");
	try {
		outputStream.write(tempMessage.getBytes());
	} catch (IOException exception) {
		System.out.println ("Network.java: IO exception: " + exception + ".");
	} catch (Exception exception) {
		System.out.println("Network.java: Unknown exception: " + exception + ".");
	}
	try {
		outputStream.flush();
	} catch (IOException exception) {
		System.out.println("Network.java: IO exception: " + exception + ".");
	} catch (Exception exception) {
		System.out.println("Network.java: Unknown exception: " + exception + ".");
	}
}

/**
 * Reads messages from the server.  When the newline character is received, the
 * preceeding line is sent to the observer.  This method blocks until it
 * receives input.
 */
public void run() {

	StringBuffer stringBuffer = new StringBuffer();
	String message;
	int i;
	char c;
	
	for (;;) {
		try {
			// Block until we get a character.
			i = inputStream.read();
		} catch (IOException exception) {
			System.out.println("Network.java: IO exception: " + exception + ".");
			break;
		} catch (Exception exception) {
			System.out.println("Network.java: Unknown exception: " + exception + ".");
			break;
		}
		if (i == -1) {
			System.out.println("Network.java: Received unexpected EOF from server.");
			break;
		}
		c = (char) i;
		if (c == '\n') {
			message = stringBuffer.toString();
			if (message.length() > 0)
				observer.processIncomingText(message);
			stringBuffer = new StringBuffer();
		} else
			stringBuffer.append(c);
	}
	System.out.println("Network.java: Exited unexpectedly.");
}

}
