/* 
 * NetworkPlayer.java 
 * 
 * Gary Givental 
 * John Gotts 
 * Saul Rodriguez 
 * 
 * EECS 481 
 * 
 * Revision History: 
 * 
 * 12/7/97 - reimplementation.
 *
 */ 
  
import java.util.*; 
 
/** 
 * Implements a network player.
 * 
 * @version     1.0 
 * @author      Gary Givental 
 * @author      John Gotts 
 * @author      Saul Rodriguez 
 */ 
 
public class NetworkPlayer extends Player { 

/** 
 * Constructs a network player.
 *
 * @param euchre	the card game.
 * @param observer	the game play observer.
 */ 
public NetworkPlayer (Euchre euchre, Observer observer, int position) { 
 
 	super (euchre, observer, position);
} 

/**
 * Picks up the trump card.
 */
public void pickUp () {

	int recipientPlayerNumber;

	recipientPlayerNumber = observer.getPlayerNumber() + position;
	if (recipientPlayerNumber > 4)
		recipientPlayerNumber -= 4;

	observer.processOutgoingText(new String("C " + recipientPlayerNumber + " pickUp"));
}

/**
 * Calls it up.
 */
public void callItUp () {

	int recipientPlayerNumber;

	recipientPlayerNumber = observer.getPlayerNumber() + position;
	if (recipientPlayerNumber > 4)
		recipientPlayerNumber -= 4;

	if (! euchre.getTable().isPassed())
		observer.processOutgoingText(new String("C " + recipientPlayerNumber + " callItUp"));
	else
		observer.processOutgoingText(new String("C " + recipientPlayerNumber + " callSuit"));

}

/** 
 * Plays the optimal card.
 *
 * @return	whether the card was legal.
 */
public boolean playCard () { 

	int recipientPlayerNumber;

	recipientPlayerNumber = observer.getPlayerNumber() + position;
	if (recipientPlayerNumber > 4)
		recipientPlayerNumber -= 4;

	observer.processOutgoingText(new String("C " + recipientPlayerNumber + " playCard"));
	return true;
}

/**
 *
 */
public Card suggestPickItUp () {

	return strategy.pickItUp ();
}

/**
 * Returns the best discard candidate.
 *
 * @return	the best discard candidate.
 */
public Card suggestDiscard () {

	return strategy.findWorst(hand.getCards(), false);
}

/**
 * Returns the optimal card.
 *
 * @return	the optimal card.
 */
public Card suggestCard () {

	return strategy.findBest(getCards());
}
 
}
