/* 
 * Player.java 
 * 
 * Gary Givental 
 * John Gotts 
 * Saul Rodriguez 
 * 
 * EECS 481 
 * 
 * Revision History: 
 * 
 * 11/24/97 - original implementation. 
 * 
 */ 
  
import java.util.*; 
 
/** 
 * Implements a player.
 * 
 * @version     1.0 
 * @author      Gary Givental 
 * @author      John Gotts 
 * @author      Saul Rodriguez 
 */ 
 
public class Player { 
 
protected Euchre euchre; 
protected Hand hand;
protected boolean dealer = false;
protected Strategy strategy;
protected Observer observer;
protected boolean winner;
protected int position;

/** 
 * Constructs a player.
 *
 * @param euchre	the card game.
 * @param observer	the game play observer.
 * @param position	the position of the player.
 */ 
public Player (Euchre euchre, Observer observer, int position) { 
 
	this.euchre = euchre; 
	this.observer = observer;
	this.position = position;
	strategy = new Strategy (euchre, observer, this);
	hand = new Hand();
	reset();
} 

public void setPosition (int position) {

	this.position = position;
}

public int getPosition () {

	return position;
}

public Hand getHand () {

	return hand;
}

/**
 * Returns the strategy engine.
 *
 * @return	the strategy engine.
 */
public Strategy getStrategy () {

	return strategy;
}

/**
 * Picks up the trump card.
 *
 * @return	the trump card.
 */
public Card pickUpTrump () {

	Card trumpCard;

	if (! dealer) {
		System.out.println("I can't pick up the trump card since I'm not the dealer.");
		System.exit(0);
	}
	if (! euchre.getTable().pickUp()) {
		System.out.println("The table was not initialized with a trump card or someone tried to pick up the trump card twice.");
		System.exit(0);
	}
	trumpCard = euchre.getTable().getTrumpCard();
	// Remove the trump card from the center frame.
	euchre.getGUI().removeCard(trumpCard, Observer.CENTER);
	hand.addCard(trumpCard);
	return trumpCard;
}

public void call () {

	observer.called(this);
}

public void callSuit (int suit) {

	observer.calledSuit(this, suit);
}

public void pass () {

	observer.passed(this);
}

/**
 * Discards the specified card.
 *
 * @param card	the card to discard.
 */
public void discardCard (Card card) {

	if (dealer) {
		hand.removeCard(card);
                observer.discardedCard(this, card);
	} else {
		System.out.println("I can't discard a card since I'm not the dealer.");
		System.exit(0);
	}
}

/**
 * Plays the specified card.
 *
 * @param card	the specified card.
 */
public void playCard (Card card) {

        if (hand.playCard(card)) {
		euchre.getGUI().addCard(card, Observer.CENTER, position);
                // Notify the observer that I played this card.
                observer.playedCard(this, card);
	} else {
                System.out.println("My deck didn't seem to have that card.");
                System.exit(0);
        }
}

/**
 * Reset my hand.
 */
public void reset () {

	hand.reset();
	winner = false;
}

/**
 * Returns whether the player is a winner.
 *
 * @return	whether the player is a winner.
 */
public boolean isWinner () {

	return winner;
}

/**
 * Sets whether the player is a winner.
 *
 * @param winner	whether the player is a winner.
 */
public void setWinner () {

	this.winner = winner;
}

/** 
 * Returns the player's cards.
 *
 * @return	the player's cards.
 */
public Vector getCards ( ) { 

	return hand.getCards(); 
}

/**
 * Returns the player's played cards.
 *
 * @return	the player's played cards.
 */
public Vector getPlayedCards () {

	return hand.getPlayedCards();
}

/** 
 * Sets whether the player is a dealer.
 *
 * @param dealer	whether the player is a dealer.
 */
public void setDealer (boolean dealer) {

	this.dealer = dealer;
}

/**
 * Returns whether the player is a dealer.
 *
 * @return	whether the player is a dealer.
 */
public boolean isDealer () {

	return dealer;
}

/**
 * Adds an arbitrary number of cards to the player's hand.
 *
 * @param newCards	the cards to be added to the player's hand.
 */
public void addCards(Vector newCards) {

	Card card;
	Enumeration e;
	int i;

	for (e = newCards.elements(); e.hasMoreElements();) {
		if (getCards().size() == 6) {
			i = position + 1;
			System.out.println("Player.java: Player " + i + " already has six cards.  No room for more!");
			System.exit(0);
		}
		card = (Card) e.nextElement();
		hand.addCard(card);
	}
}

/** 
 * META: Prints the player's cards.
 */
public void printCards () { 
	 
	Enumeration e;
	Card card; 
	 
	for (e = getCards().elements(); e.hasMoreElements();) { 
	     	card = (Card) e.nextElement(); 
		System.out.print(" " + card.getDescription());
	}
	System.out.println(".");
}	 
 
}
