/*
 * Table.java
 *
 * Gary Givental
 * John Gotts
 * Saul Rodriguez
 *
 * EECS 481
 *
 * Revision History:
 *
 * 10/05/97 - original implementation.
 *
 */

/**
 * Implements a table, analogous to an actual table.  Contains the possible
 * trump card.
 *
 * @version     1.0
 * @author      Gary Givental
 * @author      John Gotts
 * @author      Saul Rodriguez
 */
public class Table {

protected Euchre euchre;
protected Card trumpCard;
protected int trumpSuit;
protected int ourScore;
protected int theirScore;
protected boolean pickedUp;
protected boolean passed;

/**
 * Constructs a table.
 *
 * @param euchre	the card game.
 */
public Table (Euchre euchre) {

	this.euchre = euchre;
	reset();
	resetScores();
}

/**
 * Resets the table.
 */
public void reset () {

	trumpCard = null;
	pickedUp = true;
	passed = false;
	trumpSuit = Card.INVALID;
}

/**
 * Sets the trump card.
 *
 * @param the trump card.
 */
public void setTrumpCard (Card trumpCard) {

        this.trumpCard = trumpCard;
	pickedUp = false;
}

/**
 * Returns the trump card.
 *
 * @return the trump card.
 */
public Card getTrumpCard () {

	return trumpCard;
}

/**
 * Picks up the trump card, if possible.
 *
 * @return	whether the trump card was picked up successfully.
 */
public boolean pickUp () {

	if (! pickedUp) {
		trumpSuit = trumpCard.getSuit();
		pickedUp = true;
		return true;
	} else
		// Trump card already picked up or trump card never set.
		return false;
}

/**
 * Returns whether or not everyone has passed once.
 *
 * @return	whether or not everyone has passed once.
 */
public boolean isPassed () {

	return passed;
}

/**
 * Sets whether or not everyone has passed once.
 *
 * @param passed	whether or not everyone has passed once.
 */
public void setPassed (boolean passed) {

	if (this.passed != passed)
		this.passed = passed;
}

/**
 * Sets the trump suit.
 *
 * @param suit	the requested trump suit.
 */
public void setTrumpSuit (int suit) {

	if (suit == Card.DIAMOND || suit == Card.HEART || suit == Card.SPADE || suit == Card.CLUB) {
		if (passed) {
			if (trumpCard.getSuit() == trumpSuit) {
				System.out.println("Table.java: Tried to call passed suit.");
				System.exit(0);
			} else
				trumpSuit = suit;
		} else {
			System.out.println("Table.java: Tried to call trump suit on first pass.");
			System.exit(0);
		}
	} else {
		System.out.println("Table.java: Tried to set trump to an invalid suit.");
		System.exit(0);
	}
}

/**
 * Returns the trump suit.
 *
 * @return	the trump suit.
 */
public int getTrumpSuit () {

	return trumpSuit;
}

/**
 * Returns their score.
 *
 * @return	their score.
 */
public int getTheirScore () {

	return theirScore;
}

public void setTheirScore (int theirScore) {

	this.theirScore = theirScore;
}

/**
 * Returns whether the trump card was picked up.
 *
 * @return	whether the trump card was picked up.
 */
public boolean isPickedUp () {

	return pickedUp;
}

/**
 * Returns our score.
 *
 * @return	our score.
 */
public int getOurScore () {

	return ourScore;
}

public void setOurScore (int ourScore) {

	this.ourScore = ourScore;
}

/**
 * Increments our score.
 *
 * @param amount	the amount to increment our score.
 */
public void incrementOurScore (int amount) {

	ourScore = ourScore + amount;
}

/**
 * Increments their score.
 *
 * @param amount	the amount to increment their score.
 */
public void incrementTheirScore (int amount) {

	theirScore = theirScore + amount;
}

/**
 * Resets the scores.
 */
public void resetScores () {

	ourScore = 0;
	theirScore = 0;
}

}
