/*
 * Client.java
 *
 * Gary Givental
 * John Gotts
 * Saul Rodriguez
 *
 * EECS 481
 *
 * Revision History:
 *
 * 10/16/97 - original implementation.
 * 11/04/97 - improved error checking.
 *            some added functionality.
 *
 */

import java.io.*;

/**
 * Implements a client.
 *
 * @version     1.0
 * @author      Gary Givental
 * @author      John Gotts
 * @author      Saul Rodriguez
 */
public class Client extends Thread {

protected Game game = null;
protected InputStream inputStream;
protected OutputStream outputStream;

/**
 * Constructs a client with the specified input and output streams.
 *
 * @param inputStream	the input stream.
 * @param outputStream	the output stream.
 */
public Client (InputStream inputStream, OutputStream outputStream) {

	this.inputStream = inputStream;
	this.outputStream = outputStream;
}

/**
 * Returns the controlling game.
 *
 * @return	the controlling game.
 */
public Game getGame () {

	return game;
}

/**
 * Sets the controlling game.
 *
 * @param game	the controlling game.
 */
public void setGame (Game game) {

	this.game = game;
}

/**
 * Returns the client's input stream.
 *
 * @return	the client's input stream.
 */
public InputStream getInputStream () {

	return inputStream;
}

/**
 * Sets the client's input stream to the specified input stream.
 *
 * @param outputStream	the new input stream.
 */
public void setInputStream (InputStream inputStream) {

	this.inputStream = inputStream;
}

/**
 * Returns the client's output stream.
 *
 * @return	the client's output stream.
 */
public OutputStream getOutputStream () {

	return outputStream;
}

/**
 * Sets the client's output stream to the specified output stream.
 *
 * @param outputStream	the new output stream.
 */
public void setOutputStream (OutputStream outputStream) {

	this.outputStream = outputStream;
}

/**
 * Closes the client's input and output streams.
 */
public void closeStreams() {

	// We don't act on the exceptions because this method is part of the
	// shutdown process.
	try {
		inputStream.close();
	} catch (IOException exception) {
		System.out.println("IO exception: " + exception + ".");
	} catch (Exception exception) {
		System.out.println("Unknown exception: " + exception + ".");
	}
	try {
		outputStream.close();
	} catch (IOException exception) {
		System.out.println("IO exception: " + exception + ".");
	} catch (Exception exception) {
		System.out.println("Unknown exception: " + exception + ".");
	}
}

/**
 * Reads input lines on behalf of the client.  When the newline character is
 * received, the preceeding line is sent to the controlling game.  This method
 * blocks until it receives input.
 */
public void run() {

	StringBuffer stringBuffer = new StringBuffer();
	String errorMessage;
	String message;
	int i;
	char c;
	boolean status;
	
	for (;;) {
		try {
			// Block until we get a character.
			i = inputStream.read();
		} catch (IOException exception) {
			errorMessage = new String("IO exception: " + exception + ".");
			break;
		} catch (Exception exception) {
			errorMessage = new String("Unknown exception: " + exception + ".");
			break;
		}
		if (i == -1) {
			errorMessage = new String("Received unexpected EOF from client.");
			break;
		}
		c = (char) i;
		if (c == '\n') {
			message = stringBuffer.toString();
			if (message.length() > 0) {
				if (game == null) {
					errorMessage = new String("Client can't send messages without being assigned to a game.");
					break;
				} else {
					status = game.processMessage(this, message);
					if (! status) {
						errorMessage = new String("Message not accepted by the game.");
						break;
					}
				}
			} else {
				errorMessage = new String("Client won't send null messages to the game.");
				break;
			}
			stringBuffer = new StringBuffer();
		} else
			stringBuffer.append(c);
	}
	if (game == null) {
		System.out.println(errorMessage);
		closeStreams();
	} else
		// Neglect error checking here since we've already had one
		// error.
		game.processMessage(this, "E " + errorMessage);
}

}
