/*
 * Game.java
 *
 * Gary Givental
 * John Gotts
 * Saul Rodriguez
 *
 * EECS 481
 *
 * Revision History:
 *
 * 10/16/97 - original implementation.
 * 11/04/97 - simplified considerably.
 *
 */

import java.io.*;
import java.util.*;

/**
 * Implements a game.  This class handles message passing between clients.
 *
 * @version     1.0
 * @author      Gary Givental
 * @author      John Gotts
 * @author      Saul Rodriguez
 */
public class Game {

// Functions as a placeholder.
protected Object object = new Object();
protected Vector clients;
protected int players;

/**
 * Constructs a new game with the specified number of players.
 *
 * @param players	the number of players.
 */
public Game (int players) {

	this.players = players;
	clients = new Vector();
}

/**
 * Adds a client to the game.
 *
 * @param client	the client.
 *
 * @return	whether the addition was successful.
 */
public boolean addClient(Client client) {

	int index;

	if (! isFull())
		if (clients.indexOf(object) == -1) {
			clients.addElement(client);
			broadcast(clients.size() + " N" + "\n");
			return true;
		} else {
			index = clients.indexOf(object);
			clients.setElementAt(client, index);
			broadcast(clients.size() + " N" + "\n");
			return true;
		}
	else
		return false;
}

/**
 * Removes a client from the game.
 *
 * @param client	the client.
 *
 * @return	whether the deletion was successful.
 */
private boolean removeClient (Client client) {

	int index;

	if (clients.contains(client)) {
		index = clients.indexOf(client);
		// Add a placeholder.
		clients.setElementAt(object, index);
		client.closeStreams();
		return true;
	} else
		return false;
}

/**
 * Sends a broadcast message to all clients.
 *
 * @param message	the message.
 *
 * @return	whether the message was sent successfully.
 */
public boolean broadcast (String message) {

	Object object;
	String errorMessage = null;
	Enumeration e;
	Client client;

	for (e = clients.elements(); e.hasMoreElements();) {
		object = (Object) e.nextElement();
		// Make sure it's not a placeholder.
		if (object instanceof Client) {
			client = (Client) object;
			try {
				client.getOutputStream().write(message.getBytes());
			} catch (IOException exception) {
				errorMessage = new String("IO exception: " + exception + ".");
				break;
			} catch (Exception exception) {
				errorMessage = new String("Unknown exception: " + exception + ".");
				break;
			}
			try {
				client.getOutputStream().flush();
			} catch (IOException exception) {
				errorMessage = new String("IO exception: " + exception + ".");
				break;
			} catch (Exception exception) {
				errorMessage = new String("Unknown exception: " + exception + ".");
				break;
			}
		}
	}
	if (errorMessage != null) {
		System.out.println(errorMessage);
		return false;
	} else
		return true;
}

/**
 * Process a message.
 *
 * @param client	the originating client.
 * @param message	the message.
 *
 * @return	whether the message was successfully processed.
 */
public synchronized boolean processMessage (Client client, String message) {

	StringTokenizer stringTokenizer;
	String command;
	int playerNumber;

	playerNumber = clients.indexOf(client) + 1;
	// Ignore extraneous messages from clients that have already been
	// disconnected.
	if (playerNumber == 0) return true;
	stringTokenizer = new StringTokenizer(message); 
	command = stringTokenizer.nextToken();
	// My name is...
	if (command.compareTo("J") == 0) {
		broadcast(playerNumber + " " + message + "\n");
	// Say.
	} else if (command.compareTo("S") == 0) {
		broadcast(playerNumber + " " + message + "\n");
	// Start the game.
	} else if (command.compareTo("G") == 0) {
		broadcast(playerNumber + " " + message + "\n");
	// Send a control message.
	} else if (command.compareTo("C") == 0) {
		broadcast(playerNumber + " " + message + "\n");
	// Send a game update message.
	} else if (command.compareTo("U") == 0) {
		broadcast(playerNumber + " " + message + "\n");
	// Play.
	} else if (command.compareTo("P") == 0) {
		broadcast(playerNumber + " " + message + "\n");
	// Pass.
	} else if (command.compareTo("W") == 0) {
		broadcast(playerNumber + " " + message + "\n");
	// Error.
	} else if (command.compareTo("E") == 0) {
		broadcast(playerNumber + " " + message + "\n");
		removeClient(client);
	// Quit.
	} else if (command.compareTo("Q") == 0) {
		broadcast(playerNumber + " " + message + "\n");
		removeClient(client);
	} else {
		System.out.println("Invalid command.");
		return false;
	}
	return true;
}

/**
 * Returns whether the game is full.
 *
 * @return	whether the game is full.
 */
public boolean isFull () {

	if (clients.size() == players && ! clients.contains(object))
		return true;
	else
		return false;
}

}
